@@ -7,7 +7,7 @@ Nearest neighbour classifier that extracts shapee features.
 import numpy as np
 import pandas as pd
 from sktime.utils.validation.panel import check_X, check_X_y
-from sktime.utils.data_processing import from_nested_to_2d_array
+from sktime.datatypes._panel._convert import from_nested_to_2d_array
 
 # Tuning
 from sklearn.model_selection import GridSearchCV
