@@ -1,9 +1,22 @@
 # -*- coding: utf-8 -*-
-""" KNN time series classification built on sklearn KNeighborsClassifier that
-supports a range of distance measure specifically for time series. This distance
-functions are defined in cython in sktime.distances.elastic_cython. Python versions
-are in sktime.distances.elastic, but these are orders of magnitude slower.
+"""KNN time series classification.
 
+ Built on sklearn KNeighborsClassifier, this class supports a range of distance
+ measure specifically for time series. These distance functions are defined in cython
+ in sktime.distances.elastic_cython. Python versions are in sktime.distances.elastic
+ but these are orders of magnitude slower.
+
+Please note that many aspects of this class are taken from scikit-learn's
+KNeighborsTimeSeriesClassifier class with necessary changes to enable use with time
+series classification data and distance measures.
+
+todo: add a utility method to set keyword args for distance measure parameters.
+(e.g.  handle the parameter name(s) that are passed as metric_params automatically,
+depending on what distance measure is used in the classifier (e.g. know that it is w
+for dtw, c for msm, etc.). Also allow long-format specification for
+non-standard/user-defined measures e.g. set_distance_params(measure_type=None,
+param_values_to_set=None,
+param_names=None)
 """
 
 __author__ = ["Jason Lines", "TonyBagnall"]
@@ -42,26 +55,10 @@ from sktime.distances.mpdist import mpdist
 from sktime.utils.validation.panel import check_X
 from sktime.utils.validation.panel import check_X_y
 
-"""
-Please note that many aspects of this class are taken from scikit-learn's
-KNeighborsTimeSeriesClassifier
-class with necessary changes to enable use with time series classification
-data and distance measures.
-
-TO-DO: add a utility method to set keyword args for distance measure
-parameters (e.g. handle the parameter
-name(s) that are passed as metric_params automatically, depending on what
-distance measure is used in the
-classifier (e.g. know that it is w for dtw, c for msm, etc.). Also allow
-long-format specification for
-non-standard/user-defined measures
-e.g. set_distance_params(measure_type=None, param_values_to_set=None,
-param_names=None)
-"""
-
 
 class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
-    """
+    """KNN Time Series Classifier.
+
     An adapted version of the scikit-learn KNeighborsClassifier to work with
     time series data.
 
@@ -88,21 +85,34 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
     Parameters
     ----------
     n_neighbors     : int, set k for knn (default =1)
-    weights         : mechanism for weighting a vote: 'uniform', 'distance'
-    or a callable function: default ==' uniform'
+    weights         : string or callable function, optional, default ==' uniform'
+                      mechanism for weighting a vote, one of: 'uniform', 'distance'
+                      or a callable function
     algorithm       : search method for neighbours {‘auto’, ‘ball_tree’,
     ‘kd_tree’, ‘brute’}: default = 'brute'
     distance          : distance measure for time series: {'dtw','ddtw',
     'wdtw','lcss','erp','msm','twe'}: default ='dtw'
     distance_params   : dictionary for metric parameters: default = None
 
+    Examples
+    --------
+    >>> from sktime.classification.distance_based import KNeighborsTimeSeriesClassifier
+    >>> from sktime.datasets import load_basic_motions
+    >>> X_train, y_train = load_basic_motions(return_X_y=True, split="train")
+    >>> X_test, y_test = load_basic_motions(return_X_y=True, split="test")
+    >>> classifier = KNeighborsTimeSeriesClassifier()
+    >>> classifier.fit(X_train, y_train)
+    KNeighborsTimeSeriesClassifier(...)
+    >>> y_pred = classifier.predict(X_test)
     """
 
-    # Capabilities: data types this classifier can handle
+    # Capability tags
     capabilities = {
         "multivariate": True,
         "unequal_length": False,
         "missing_values": False,
+        "train_estimate": False,
+        "contractable": False,
     }
 
     def __init__(
@@ -175,7 +185,7 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         self._is_fitted = False
 
     def fit(self, X, y):
-        """Fit the model using X as training data and y as target values
+        """Fit the model using X as training data and y as target values.
 
         Parameters
         ----------
@@ -184,7 +194,6 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
 
         y : {array-like, sparse matrix}
             Target values of shape = [n_samples]
-
         """
         X, y = check_X_y(
             X,
@@ -256,28 +265,27 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         return fx
 
     def _more_tags(self):
-        """Removes the need to pass y with _fit
+        """Remove the need to pass y with _fit.
+
         Overrides the scikit learn (>0.23) base class setting where 'requires_y' is true
         so we can call fx = self._fit(X) and maintain backward compatibility.
         """
         return {"requires_y": False}
 
     def kneighbors(self, X, n_neighbors=None, return_distance=True):
-        """Finds the K-neighbors of a point.
+        """Find the K-neighbors of a point.
+
         Returns indices of and distances to the neighbors of each point.
 
         Parameters
         ----------
         X : sktime-format pandas dataframe with shape([n_cases,n_dimensions]),
         or numpy ndarray with shape([n_cases,n_readings,n_dimensions])
-
         y : {array-like, sparse matrix}
             Target values of shape = [n_samples]
-
         n_neighbors : int
             Number of neighbors to get (default is the value
             passed to the constructor).
-
         return_distance : boolean, optional. Defaults to True.
             If False, distances will not be returned
 
@@ -286,7 +294,6 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         dist : array
             Array representing the lengths to points, only present if
             return_distance=True
-
         ind : array
             Indices of the nearest points in the population matrix.
         """
@@ -390,14 +397,12 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
             return neigh_ind
 
     def predict(self, X):
-        """Predict the class labels for the provided data
+        """Predict the class labels for the provided data.
 
         Parameters
         ----------
         X : sktime-format pandas dataframe or array-like, shape (n_query,
-        n_features), \
-                or (n_query, n_indexed) if metric == 'precomputed'
-            Test samples.
+        n_features), or (n_query, n_indexed) if metric == 'precomputed' test samples.
 
         Returns
         -------
