@@ -566,8 +566,7 @@ class _DirRecReducer(_Reducer):
 
 
 class DirectTabularRegressionForecaster(_DirectReducer):
-    """
-    Direct reduction from forecasting to tabular regression.
+    """Direct reduction from forecasting to tabular regression.
 
     For the direct reduction strategy, a separate forecaster is fitted
     for each step ahead of the forecasting horizon.
@@ -585,8 +584,7 @@ class DirectTabularRegressionForecaster(_DirectReducer):
 
 
 class MultioutputTabularRegressionForecaster(_MultioutputReducer):
-    """
-    Multioutput reduction from forecasting to tabular regression.
+    """Multioutput reduction from forecasting to tabular regression.
 
     For the multioutput strategy, a single estimator capable of handling multioutput
     targets is fitted to all the future steps in the forecasting horizon.
@@ -604,8 +602,7 @@ class MultioutputTabularRegressionForecaster(_MultioutputReducer):
 
 
 class RecursiveTabularRegressionForecaster(_RecursiveReducer):
-    """
-    Recursive reduction from forecasting to tabular regression.
+    """Recursive reduction from forecasting to tabular regression.
 
     For the recursive strategy, a single estimator is fit for a one-step-ahead
     forecasting horizon and then called iteratively to predict multiple steps ahead.
@@ -623,8 +620,7 @@ class RecursiveTabularRegressionForecaster(_RecursiveReducer):
 
 
 class DirRecTabularRegressionForecaster(_DirRecReducer):
-    """
-    Dir-rec reduction from forecasting to tabular regression.
+    """Dir-rec reduction from forecasting to tabular regression.
 
     For the hybrid dir-rec strategy, a separate forecaster is fitted
     for each step ahead of the forecasting horizon and then
@@ -645,8 +641,7 @@ class DirRecTabularRegressionForecaster(_DirRecReducer):
 
 
 class DirectTimeSeriesRegressionForecaster(_DirectReducer):
-    """
-    Direct reduction from forecasting to time-series regression.
+    """Direct reduction from forecasting to time-series regression.
 
     For the direct reduction strategy, a separate forecaster is fitted
     for each step ahead of the forecasting horizon.
@@ -664,8 +659,7 @@ class DirectTimeSeriesRegressionForecaster(_DirectReducer):
 
 
 class MultioutputTimeSeriesRegressionForecaster(_MultioutputReducer):
-    """
-    Multioutput reduction from forecasting to time series regression.
+    """Multioutput reduction from forecasting to time series regression.
 
     For the multioutput strategy, a single estimator capable of handling multioutput
     targets is fitted to all the future steps in the forecasting horizon.
@@ -683,8 +677,7 @@ class MultioutputTimeSeriesRegressionForecaster(_MultioutputReducer):
 
 
 class RecursiveTimeSeriesRegressionForecaster(_RecursiveReducer):
-    """
-    Recursive reduction from forecasting to time series regression.
+    """Recursive reduction from forecasting to time series regression.
 
     For the recursive strategy, a single estimator is fit for a one-step-ahead
     forecasting horizon and then called iteratively to predict multiple steps ahead.
@@ -702,8 +695,7 @@ class RecursiveTimeSeriesRegressionForecaster(_RecursiveReducer):
 
 
 class DirRecTimeSeriesRegressionForecaster(_DirRecReducer):
-    """
-    Dir-rec reduction from forecasting to time-series regression.
+    """Dir-rec reduction from forecasting to time-series regression.
 
     For the hybrid dir-rec strategy, a separate forecaster is fitted
     for each step ahead of the forecasting horizon and then
@@ -727,8 +719,7 @@ class DirRecTimeSeriesRegressionForecaster(_DirRecReducer):
 def ReducedForecaster(
     estimator, scitype="infer", strategy="recursive", window_length=10, step_length=1
 ):
-    """
-    Reduction from forecasting to tabular or time series regression.
+    """Reduction from forecasting to tabular or time series regression.
 
     During fitting, a sliding-window approach is used to first transform the
     time series into tabular or panel data, which is then used to fit a tabular or
@@ -765,8 +756,7 @@ def ReducedForecaster(
 def ReducedRegressionForecaster(
     estimator, scitype, strategy="recursive", window_length=10, step_length=1
 ):
-    """
-    Reduction from forecasting to tabular or time series regression.
+    """Reduction from forecasting to tabular or time series regression.
 
     During fitting, a sliding-window approach is used to first transform the
     time series into tabular or panel data, which is then used to fit a tabular or
@@ -804,8 +794,7 @@ def make_reduction(
     window_length=10,
     scitype="infer",
 ):
-    """
-    Make forecaster based on reduction to tabular or time-series regression.
+    """Make forecaster based on reduction to tabular or time-series regression.
 
     During fitting, a sliding-window approach is used to first transform the
     time series into tabular or panel data, which is then used to fit a tabular or
@@ -825,6 +814,7 @@ def make_reduction(
     scitype : str, optional (default="infer")
         Must be one of "infer", "tabular-regressor" or "time-series-regressor". If
         the scitype cannot be inferred, please specify it explicitly.
+        See :term:`scitype`.
 
     Returns
     -------
