@@ -1,6 +1,7 @@
 #!/usr/bin/env python3 -u
 # -*- coding: utf-8 -*-
 # copyright: sktime developers, BSD-3-Clause License (see LICENSE file)
+"""Implements Prophet forecaster by wrapping fbprophet."""
 
 __author__ = ["Martin Walter"]
 __all__ = ["Prophet"]
@@ -14,13 +15,14 @@ _check_soft_dependencies("fbprophet")
 
 class Prophet(_ProphetAdapter):
     """Prophet forecaster by wrapping fbprophet.
+
     Parameters
     ----------
     freq: String of DatetimeIndex frequency. See here for possible values:
         https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html
         #timeseries-offset-aliases
-    add_seasonality: Dict with args for Prophet.add_seasonality().
-        Dict can have the following keys/values:
+    add_seasonality: Dict or List of Dicts with args for Prophet.add_seasonality().
+        Dict or each Dict in List can have the following keys/values:
             name: string name of the seasonality component.
             period: float number of days in one period.
             fourier_order: int number of Fourier components to use.
@@ -82,6 +84,20 @@ class Prophet(_ProphetAdapter):
     https://facebook.github.io/prophet
     https://github.com/facebook/prophet
 
+    Examples
+    --------
+    >>> from sktime.datasets import load_airline
+    >>> from sktime.forecasting.fbprophet import Prophet
+    >>> # Prophet requires to have data with a pandas.DatetimeIndex
+    >>> y = load_airline().to_timestamp(freq='M')
+    >>> forecaster = Prophet(
+    ...     seasonality_mode='multiplicative',
+    ...     n_changepoints=int(len(y) / 12),
+    ...     add_country_holidays={'country_name': 'Germany'},
+    ...     yearly_seasonality=True)
+    >>> forecaster.fit(y)
+    Prophet(...)
+    >>> y_pred = forecaster.predict(fh=[1,2,3])
     """
 
     def __init__(
@@ -122,9 +138,9 @@ class Prophet(_ProphetAdapter):
         self.daily_seasonality = daily_seasonality
         self.holidays = holidays
         self.seasonality_mode = seasonality_mode
-        self.seasonality_prior_scale = float(seasonality_prior_scale)
-        self.changepoint_prior_scale = float(changepoint_prior_scale)
-        self.holidays_prior_scale = float(holidays_prior_scale)
+        self.seasonality_prior_scale = seasonality_prior_scale
+        self.changepoint_prior_scale = changepoint_prior_scale
+        self.holidays_prior_scale = holidays_prior_scale
         self.mcmc_samples = mcmc_samples
         self.alpha = alpha
         self.uncertainty_samples = uncertainty_samples
@@ -149,9 +165,9 @@ class Prophet(_ProphetAdapter):
             daily_seasonality=self.daily_seasonality,
             holidays=self.holidays,
             seasonality_mode=self.seasonality_mode,
-            seasonality_prior_scale=self.seasonality_prior_scale,
-            holidays_prior_scale=self.holidays_prior_scale,
-            changepoint_prior_scale=self.changepoint_prior_scale,
+            seasonality_prior_scale=float(self.seasonality_prior_scale),
+            holidays_prior_scale=float(self.holidays_prior_scale),
+            changepoint_prior_scale=float(self.changepoint_prior_scale),
             mcmc_samples=self.mcmc_samples,
             interval_width=1 - self.alpha,
             uncertainty_samples=self.uncertainty_samples,
