@@ -1,6 +1,7 @@
 #!/usr/bin/env python3 -u
 # -*- coding: utf-8 -*-
 # copyright: sktime developers, BSD-3-Clause License (see LICENSE file)
+"""Implement dataset splitting for model evaluation and seleciton."""
 
 __all__ = [
     "ExpandingWindowSplitter",
@@ -34,7 +35,7 @@ DEFAULT_FH = 1
 
 
 def _repr(self):
-    """Helper function to build repr for splitters similar to estimator objects"""
+    """Build repr for splitters similar to estimator objects."""
     # This is copied from scikit-learn's BaseEstimator get_params method
     cls = self.__class__
     init = getattr(cls.__init__, "deprecated_original", cls.__init__)
@@ -96,20 +97,19 @@ def _repr(self):
 
 
 def _check_y(y):
-    """Check input to `split` function"""
+    """Check input to `split` function."""
     if isinstance(y, pd.Series):
         y = y.index
     return check_time_index(y)
 
 
 def _check_fh(fh):
-    """Check and convert fh to format expected by CV splitters"""
+    """Check and convert fh to format expected by CV splitters."""
     return check_fh(fh, enforce_relative=True)
 
 
 def _get_end(y, fh):
-    """Compute the end of the last training window for a given and forecasting
-    horizon."""
+    """Compute the end of the last training window for a forecasting horizon."""
     # `fh` is assumed to be ordered and checked by `_check_fh` and `window_length` by
     # `check_window_length`.
     n_timepoints = y.shape[0]
@@ -182,7 +182,7 @@ class BaseSplitter:
             yield train[train >= 0], test[test >= 0]
 
     def _split(self, y):
-        """Internal split method implemented by concrete classes"""
+        """Split method containing internal logic implemented by concrete classes."""
         raise NotImplementedError("abstract method")
 
     def get_n_splits(self, y=None):
@@ -216,7 +216,7 @@ class BaseSplitter:
         raise NotImplementedError("abstract method")
 
     def get_fh(self):
-        """Return the forecasting horizon
+        """Return the forecasting horizon.
 
         Returns
         -------
@@ -266,16 +266,16 @@ class CutoffSplitter(BaseSplitter):
             yield training_window, test_window
 
     def get_n_splits(self, y=None):
-        """Return the number of splits"""
+        """Return the number of splits."""
         return len(self.cutoffs)
 
     def get_cutoffs(self, y=None):
-        """Return the cutoff points"""
+        """Return the cutoff points."""
         return check_cutoffs(self.cutoffs)
 
 
 class BaseWindowSplitter(BaseSplitter):
-    """Base class for sliding and expanding window splitter"""
+    """Base class for sliding and expanding window splitter."""
 
     def __init__(
         self,
@@ -333,12 +333,11 @@ class BaseWindowSplitter(BaseSplitter):
 
     @staticmethod
     def _split_windows(start, end, step_length, window_length, fh):
-        """Abstract method implemented by concrete classes for sliding and expanding
-        windows"""
+        """Abstract method for sliding/expanding windows."""
         raise NotImplementedError("abstract method")
 
     def _get_start(self, fh):
-        """Get the first split point"""
+        """Get the first split point."""
         # By default, the first split point is the index zero, the first
         # observation in
         # the data.
@@ -369,7 +368,7 @@ class BaseWindowSplitter(BaseSplitter):
         return start
 
     def get_n_splits(self, y=None):
-        """Return number of splits
+        """Return number of splits.
 
         Parameters
         ----------
@@ -467,7 +466,7 @@ class SlidingWindowSplitter(BaseWindowSplitter):
 
     @staticmethod
     def _split_windows(start, end, step_length, window_length, fh):
-        """Sliding windows"""
+        """Generate sliding windows."""
         for split_point in range(start, end, step_length):
             train = np.arange(split_point - window_length, split_point)
             test = split_point + fh - 1
@@ -526,7 +525,7 @@ class ExpandingWindowSplitter(BaseWindowSplitter):
 
     @staticmethod
     def _split_windows(start, end, step_length, window_length, fh):
-        """Expanding windows"""
+        """Generate expanding windows."""
         for split_point in range(start, end, step_length):
             train = np.arange(start - window_length, split_point)
             test = split_point + fh - 1
@@ -594,7 +593,8 @@ class SingleWindowSplitter(BaseSplitter):
 
 
 def temporal_train_test_split(y, X=None, test_size=None, train_size=None, fh=None):
-    """Split arrays or matrices into sequential train and test subsets
+    """Split arrays or matrices into sequential train and test subsets.
+
     Creates train/test splits over endogenous arrays an optional exogenous
     arrays.
 
@@ -648,8 +648,10 @@ def temporal_train_test_split(y, X=None, test_size=None, train_size=None, fh=Non
 
 
 def _split_by_fh(y, fh, X=None):
-    """Helper function to split time series with forecasting horizon handling both
-    relative and absolute horizons"""
+    """Split time series with forecasting horizon.
+
+    Handles both relative and absolute horizons.
+    """
     if X is not None:
         check_equal_time_index(y, X)
     fh = check_fh(fh)
