@@ -1,27 +1,26 @@
 # -*- coding: utf-8 -*-
+"""Functions for the input and output of data and results."""
+
 import itertools
 import os
 import textwrap
 
 import numpy as np
 import pandas as pd
-from sklearn.metrics import accuracy_score as acc
-from sktime.utils.data_processing import _make_column_names, from_long_to_nested
+
+from sktime.datatypes._panel._convert import _make_column_names, from_long_to_nested
+from sktime.transformations.base import BaseTransformer
+from sktime.utils.validation.panel import check_X, check_X_y
 
 
 class TsFileParseException(Exception):
-    """
-    Should be raised when parsing a .ts file and the format is incorrect.
-    """
+    """Should be raised when parsing a .ts file and the format is incorrect."""
 
     pass
 
 
 class LongFormatDataParseException(Exception):
-    """
-    Should be raised when parsing a .csv file
-    with long-formatted date and the format is incorrect
-    """
+    """Should be raised when parsing a .csv file with long-formatted data."""
 
     pass
 
@@ -31,7 +30,7 @@ def load_from_tsfile_to_dataframe(
     return_separate_X_and_y=True,
     replace_missing_vals_with="NaN",
 ):
-    """Loads data from a .ts file into a Pandas DataFrame.
+    """Load data from a .ts file into a Pandas DataFrame.
 
     Parameters
     ----------
@@ -56,9 +55,7 @@ def load_from_tsfile_to_dataframe(
         all time-series and (if relevant) a column "class_vals" the
         associated class values.
     """
-
     # Initialize flags and variables used when parsing the file
-
     metadata_started = False
     data_started = False
 
@@ -746,7 +743,7 @@ def load_from_arff_to_dataframe(
     return_separate_X_and_y=True,
     replace_missing_vals_with="NaN",
 ):
-    """Loads data from a .ts file into a Pandas DataFrame.
+    """Load data from a .ts file into a Pandas DataFrame.
 
     Parameters
     ----------
@@ -775,7 +772,6 @@ def load_from_arff_to_dataframe(
         all time-series and (if relevant) a column "class_vals" the
         associated class values.
     """
-
     instance_list = []
     class_val_list = []
 
@@ -863,7 +859,7 @@ def load_from_arff_to_dataframe(
 def load_from_ucr_tsv_to_dataframe(
     full_file_path_and_name, return_separate_X_and_y=True
 ):
-    """Loads data from a .tsv file into a Pandas DataFrame.
+    """Load data from a .tsv file into a Pandas DataFrame.
 
     Parameters
     ----------
@@ -885,7 +881,6 @@ def load_from_ucr_tsv_to_dataframe(
         all time-series and (if relevant) a column "class_vals" the
         associated class values.
     """
-
     df = pd.read_csv(full_file_path_and_name, sep="\t", header=None)
     y = df.pop(0).values
     df.columns -= 1
@@ -898,7 +893,7 @@ def load_from_ucr_tsv_to_dataframe(
 
 
 def load_from_long_to_dataframe(full_file_path_and_name, separator=","):
-    """Loads data from a long format file into a Pandas DataFrame.
+    """Load data from a long format file into a Pandas DataFrame.
 
     Parameters
     ----------
@@ -912,7 +907,6 @@ def load_from_long_to_dataframe(full_file_path_and_name, separator=","):
     DataFrame
         A dataframe with sktime-formatted data
     """
-
     data = pd.read_csv(full_file_path_and_name, sep=separator, header=0)
     # ensure there are 4 columns in the long_format table
     if len(data.columns) != 4:
@@ -935,7 +929,7 @@ def load_from_long_to_dataframe(full_file_path_and_name, separator=","):
 
 # left here for now, better elsewhere later perhaps
 def generate_example_long_table(num_cases=50, series_len=20, num_dims=2):
-    """Generates example from long table format file.
+    """Generate example from long table format file.
 
     Parameters
     ----------
@@ -950,7 +944,6 @@ def generate_example_long_table(num_cases=50, series_len=20, num_dims=2):
     -------
     DataFrame
     """
-
     rows_per_case = series_len * num_dims
     total_rows = num_cases * series_len * num_dims
 
@@ -974,7 +967,7 @@ def generate_example_long_table(num_cases=50, series_len=20, num_dims=2):
 
 
 def make_multi_index_dataframe(n_instances=50, n_columns=3, n_timepoints=20):
-    """Generates example multi-index DataFrame.
+    """Generate example multi-index DataFrame.
 
     Parameters
     ----------
@@ -993,7 +986,6 @@ def make_multi_index_dataframe(n_instances=50, n_columns=3, n_timepoints=20):
         The multi-indexed DataFrame with
         shape (n_instances*n_timepoints, n_column).
     """
-
     # Make long DataFrame
     long_df = generate_example_long_table(
         num_cases=n_instances, series_len=n_timepoints, num_dims=n_columns
@@ -1006,33 +998,67 @@ def make_multi_index_dataframe(n_instances=50, n_columns=3, n_timepoints=20):
 
 
 def write_results_to_uea_format(
-    path,
-    strategy_name,
+    estimator_name,
     dataset_name,
-    y_true,
     y_pred,
+    output_path,
+    full_path=True,
+    y_true=None,
+    predicted_probs=None,
     split="TEST",
     resample_seed=0,
-    y_proba=None,
-    second_line="N/A",
+    second_line="No Parameter Info",
+    third_line="N/A",
 ):
+    """Write the predictions for an experiment in the standard format used by sktime.
+
+    Parameters
+    ----------
+    estimator_name : str,
+        Name of the object that made the predictions, written to file and can
+        deterimine file structure of output_root is True
+    dataset_name : str
+        name of the problem the classifier was built on
+    y_pred : np.array
+        predicted values
+    output_path : str
+        Path where to put results. Either a root path, or a full path
+    full_path : boolean, default = True
+        If False, then the standard file structure is created. If false, results are
+        written directly to the directory passed in output_path
+    y_true : np.array, default = None
+        Actual values, written to file with the predicted values if present
+    predicted_probs :  np.ndarray, default = None
+        Estimated class probabilities. If passed, these are written after the
+        predicted values. Regressors should not pass anything
+    split : str, default = "TEST"
+        Either TRAIN or TEST, depending on the results, influences file name.
+    resample_seed : int, default = 0
+        Indicates what data
+    second_line : str
+        unstructured, used for predictor parameters
+    third_line : str
+        summary performance information (see comment below)
+    """
     if len(y_true) != len(y_pred):
         raise IndexError(
-            "The number of predicted class values is not the same as the "
+            "The number of predicted values is not the same as the "
             "number of actual class values"
         )
-
-    try:
-        os.makedirs(
-            str(path)
+    # If the full directory path is not passed, make the standard structure
+    if not full_path:
+        output_path = (
+            str(output_path)
             + "/"
-            + str(strategy_name)
+            + str(estimator_name)
             + "/Predictions/"
             + str(dataset_name)
             + "/"
         )
+    try:
+        os.makedirs(output_path)
     except os.error:
-        pass  # raises os.error if path already exists
+        pass  # raises os.error if path already exists, so just ignore this
 
     if split == "TRAIN" or split == "train":
         train_or_test = "train"
@@ -1042,65 +1068,174 @@ def write_results_to_uea_format(
         raise ValueError("Unknown 'split' value - should be TRAIN/train or TEST/test")
 
     file = open(
-        str(path)
-        + "/"
-        + str(strategy_name)
-        + "/Predictions/"
-        + str(dataset_name)
+        str(output_path)
         + "/"
         + str(train_or_test)
-        + "Fold"
+        + "Resample"
         + str(resample_seed)
         + ".csv",
         "w",
     )
 
-    correct = acc(y_true, y_pred)
-
     # the first line of the output file is in the form of:
     # <classifierName>,<datasetName>,<train/test>
     file.write(
-        str(strategy_name) + "," + str(dataset_name) + "," + str(train_or_test) + "\n"
+        str(estimator_name) + "," + str(dataset_name) + "," + str(train_or_test) + "\n"
     )
 
-    # the second line of the output is free form and classifier-specific;
-    # usually this will record info
-    # such as build time, paramater options used, any constituent model
+    # the second line of the output is free form and estimator-specific; usually this
+    # will record info such as build time, paramater options used, any constituent model
     # names for ensembles, etc.
     file.write(str(second_line) + "\n")
 
     # the third line of the file is the accuracy (should be between 0 and 1
-    # inclusive). If this is a train
-    # output file then it will be a training estimate of the classifier on
-    # the training data only (e.g.
-    # 10-fold cv, leave-one-out cv, etc.). If this is a test output file,
-    # it should be the output
-    # of the estimator on the test data (likely trained on the training data
-    # for a-priori parameter optimisation)
+    # inclusive). If this is a train output file then it will be a training estimate
+    # of the classifier on the training data only (e.g. 10-fold cv, leave-one-out cv,
+    # etc.). If this is a test output file, it should be the output of the estimator
+    # on the test data (likely trained on the training data for a-priori parameter
+    # optimisation)
+    file.write(str(third_line) + "\n")
 
-    file.write(str(correct) + "\n")
-
-    # from line 4 onwards each line should include the actual and predicted
-    # class labels (comma-separated). If
-    # present, for each case, the probabilities of predicting every class
-    # value for this case should also be
-    # appended to the line (a space is also included between the predicted
-    # value and the predict_proba). E.g.:
+    # from line 4 onwards each line should include the actual and predicted class
+    # labels (comma-separated). If present, for each case, the probabilities of
+    # predicting every class value for this case should also be appended to the line (
+    # a space is also included between the predicted value and the predict_proba). E.g.:
     #
     # if predict_proba data IS provided for case i:
-    #   actual_class_val[i], predicted_class_val[i],,prob_class_0[i],
+    #   y_true[i], y_pred[i],,prob_class_0[i],
     #   prob_class_1[i],...,prob_class_c[i]
     #
     # if predict_proba data IS NOT provided for case i:
-    #   actual_class_val[i], predicted_class_val[i]
-    for i in range(0, len(y_pred)):
-        file.write(str(y_true[i]) + "," + str(y_pred[i]))
-        if y_proba is not None:
-            file.write(",")
-            for j in y_proba[i]:
-                file.write("," + str(j))
-            file.write("\n")  # TODO BUG new line is written only if the
-            # probas are provided!!!!
+    #   y_true[i], y_pred[i]
+    # If y_true is None (if clustering), y_true[i] is replaced with ? to indicate
+    # missing
+    if y_true is None:
+        for i in range(0, len(y_pred)):
+            file.write("?," + str(y_pred[i]))
+            if predicted_probs is not None:
+                file.write(",")
+                for j in predicted_probs[i]:
+                    file.write("," + str(j))
+            file.write("\n")
+    else:
+        for i in range(0, len(y_pred)):
+            file.write(str(y_true[i]) + "," + str(y_pred[i]))
+            if predicted_probs is not None:
+                file.write(",")
+                for j in predicted_probs[i]:
+                    file.write("," + str(j))
+            file.write("\n")
+    file.close()
+
+
+def write_tabular_transformation_to_arff(
+    data,
+    transformation,
+    path,
+    problem_name="sample_data",
+    class_label=None,
+    class_value_list=None,
+    comment=None,
+    fold="",
+    fit_transform=True,
+):
+    """
+    Transform a dataset using a tabular transformer and write the result to a arff file.
+
+    Parameters
+    ----------
+    data: pandas dataframe or 3d numpy array
+        The dataset to build the transformation with which must be of the structure
+        specified in the documentation examples/loading_data.ipynb.
+    transformation: BaseTransformer
+        Transformation use and to save to arff.
+    path: str
+        The full path to output the arff file to.
+    problem_name: str, default="sample_data"
+        The problemName to print in the header of the arff file and also the name of
+        the file.
+    class_label: list of str or None, default=None
+        The problems class labels to show the possible class values for in the file
+        header, optional.
+    class_value_list: list, ndarray or None, default=None
+        The class values for each case, optional.
+    comment: str or None, default=None
+        Comment text to be inserted before the header in a block.
+    fold: str or None, default=None
+        Addon at the end of the filename, i.e. _TRAIN or _TEST.
+    fit_transform: bool, default=True
+        Whether to fit the transformer prior to calling transform.
+
+    Returns
+    -------
+    None
+    """
+    # ensure transformation provided is a transformer
+    if not isinstance(transformation, BaseTransformer):
+        raise ValueError("Transformation must be a BaseTransformer")
+
+    if fit_transform:
+        data = transformation.fit_transform(data, class_value_list)
+    else:
+        data = transformation.transform(data, class_value_list)
+
+    if isinstance(data, pd.DataFrame):
+        data = data.to_numpy()
+
+    if class_value_list is not None and class_label is None:
+        class_label = np.unique(class_value_list)
+    elif class_value_list is None:
+        class_value_list = []
+
+    # ensure number of cases is same as the class value list
+    if len(data) != len(class_value_list) and len(class_value_list) > 0:
+        raise IndexError(
+            "The number of cases is not the same as the number of given class values"
+        )
+
+    if fold is None:
+        fold = ""
+
+    # create path if not exist
+    dirt = f"{str(path)}/{str(problem_name)}-{type(transformation).__name__}/"
+    try:
+        os.makedirs(dirt)
+    except os.error:
+        pass  # raises os.error if path already exists
+
+    # create arff file in the path
+    file = open(
+        f"{dirt}{str(problem_name)}-{type(transformation).__name__}{fold}.arff", "w"
+    )
+
+    # write comment if any as a block at start of file
+    if comment is not None:
+        file.write("\n% ".join(textwrap.wrap("% " + comment)))
+        file.write("\n")
+
+    # begin writing header information
+    file.write(f"@Relation {problem_name}\n")
+
+    # write each attribute
+    for i in range(data.shape[1]):
+        file.write(f"@attribute att{str(i)} numeric\n")
+
+    # write class attribute if it exists
+    if class_label is not None:
+        comma_separated_class_label = ",".join(str(label) for label in class_label)
+        file.write(f"@attribute target {{{comma_separated_class_label}}}\n")
+
+    file.write("@data\n")
+
+    for case, value in itertools.zip_longest(data, class_value_list):
+        # turn attributes into comma-separated row
+        atts = ",".join([str(num) if not np.isnan(num) else "?" for num in case])
+        file.write(str(atts))
+        if value is not None:
+            file.write(f",{value}")  # write the case value if any
+        elif class_label is not None:
+            file.write(",?")
+        file.write("\n")  # open a new line
 
     file.close()
 
@@ -1109,52 +1244,48 @@ def write_dataframe_to_tsfile(
     data,
     path,
     problem_name="sample_data",
-    timestamp=False,
-    univariate=True,
     class_label=None,
     class_value_list=None,
     equal_length=False,
     series_length=-1,
     missing_values="NaN",
     comment=None,
+    fold="",
 ):
     """
-    Output a dataset in dataframe format to .ts file
+    Output a dataset in dataframe format to .ts file.
+
     Parameters
     ----------
     data: pandas dataframe
-        the dataset in a dataframe to be written as a ts file
+        The dataset in a dataframe to be written as a ts file
         which must be of the structure specified in the documentation
-        https://github.com/whackteachers/sktime/blob/master/examples/loading_data.ipynb
+        examples/loading_data.ipynb.
         index |   dim_0   |   dim_1   |    ...    |  dim_c-1
            0  | pd.Series | pd.Series | pd.Series | pd.Series
            1  | pd.Series | pd.Series | pd.Series | pd.Series
           ... |    ...    |    ...    |    ...    |    ...
            n  | pd.Series | pd.Series | pd.Series | pd.Series
     path: str
-        The full path to output the ts file
-    problem_name: str
-        The problemName to print in the header of the ts file
-        and also the name of the file.
-    timestamp: {False, bool}, optional
-        Indicate whether the data contains timestamps in the header.
-    univariate: {True, bool}, optional
-        Indicate whether the data is univariate or multivariate in the header.
-        If univariate, only the first dimension will be written to file
-    class_label: {list, None}, optional
-        Provide class label to show the possible class values
-        for classification problems in the header.
-    class_value_list: {list/ndarray, []}, optional
-        ndarray containing the class values for each case in classification problems
-    equal_length: {False, bool}, optional
-        Indicate whether each series has equal length. It only write to file if true.
-    series_length: {-1, int}, optional
-        Indicate each series length if they are of equal length.
-        It only write to file if true.
-    missing_values: {NaN, str}, optional
-        Representation for missing value, default is NaN.
-    comment: {None, str}, optional
+        The full path to output the ts file to.
+    problem_name: str, default="sample_data"
+        The problemName to print in the header of the ts file and also the name of
+        the file.
+    class_label: list of str or None, default=None
+        The problems class labels to show the possible class values for in the file
+        header, optional.
+    class_value_list: list, ndarray or None, default=None
+        The class values for each case, optional.
+    equal_length: bool, default=False
+        Indicates whether each series is of equal length.
+    series_length: int, default=-1
+        Indicates the series length if they are of equal length.
+    missing_values: str, default="NaN"
+        Representation for missing values.
+    comment: str or None, default=None
         Comment text to be inserted before the header in a block.
+    fold: str or None, default=None
+        Addon at the end of the filename, i.e. _TRAIN or _TEST.
 
     Returns
     -------
@@ -1163,22 +1294,102 @@ def write_dataframe_to_tsfile(
     Notes
     -----
     This version currently does not support writing timestamp data.
-
-    References
-    ----------
-    The code for writing series data into file is adopted from
-    https://stackoverflow.com/questions/37877708/
-    how-to-turn-a-pandas-dataframe-row-into-a-comma-separated-string
     """
-    if class_value_list is None:
-        class_value_list = []
     # ensure data provided is a dataframe
     if not isinstance(data, pd.DataFrame):
         raise ValueError("Data provided must be a DataFrame")
+
+    if class_value_list is not None:
+        data, class_value_list = check_X_y(data, class_value_list, coerce_to_numpy=True)
+    else:
+        data = check_X(data, coerce_to_numpy=True)
+
+    # ensure data provided is a dataframe
+    write_ndarray_to_tsfile(
+        data,
+        path,
+        problem_name=problem_name,
+        class_label=class_label,
+        class_value_list=class_value_list,
+        equal_length=equal_length,
+        series_length=series_length,
+        missing_values=missing_values,
+        comment=comment,
+        fold=fold,
+    )
+
+
+def write_ndarray_to_tsfile(
+    data,
+    path,
+    problem_name="sample_data",
+    class_label=None,
+    class_value_list=None,
+    equal_length=False,
+    series_length=-1,
+    missing_values="NaN",
+    comment=None,
+    fold="",
+):
+    """
+    Output a dataset in ndarray format to .ts file.
+
+    Parameters
+    ----------
+    data: pandas dataframe
+        The dataset in a 3d ndarray to be written as a ts file
+        which must be of the structure specified in the documentation
+        examples/loading_data.ipynb.
+        (n_instances, n_columns, n_timepoints)
+    path: str
+        The full path to output the ts file to.
+    problem_name: str, default="sample_data"
+        The problemName to print in the header of the ts file and also the name of
+        the file.
+    class_label: list of str or None, default=None
+        The problems class labels to show the possible class values for in the file
+        header.
+    class_value_list: list, ndarray or None, default=None
+        The class values for each case, optional.
+    equal_length: bool, default=False
+        Indicates whether each series is of equal length.
+    series_length: int, default=-1
+        Indicates the series length if they are of equal length.
+    missing_values: str, default="NaN"
+        Representation for missing values.
+    comment: str or None, default=None
+        Comment text to be inserted before the header in a block.
+    fold: str or None, default=None
+        Addon at the end of the filename, i.e. _TRAIN or _TEST.
+
+    Returns
+    -------
+    None
+
+    Notes
+    -----
+    This version currently does not support writing timestamp data.
+    """
+    # ensure data provided is a ndarray
+    if not isinstance(data, np.ndarray):
+        raise ValueError("Data provided must be a ndarray")
+
+    if class_value_list is not None:
+        data, class_value_list = check_X_y(data, class_value_list)
+    else:
+        data = check_X(data)
+
+    univariate = data.shape[1] == 1
+
+    if class_value_list is not None and class_label is None:
+        class_label = np.unique(class_value_list)
+    elif class_value_list is None:
+        class_value_list = []
+
     # ensure number of cases is same as the class value list
-    if len(data.index) != len(class_value_list) and len(class_value_list) > 0:
+    if len(data) != len(class_value_list) and len(class_value_list) > 0:
         raise IndexError(
-            "The number of cases is not the same as the number of given " "class values"
+            "The number of cases is not the same as the number of given class values"
         )
 
     if equal_length and series_length == -1:
@@ -1186,6 +1397,9 @@ def write_dataframe_to_tsfile(
             "Please specify the series length for equal length time series data."
         )
 
+    if fold is None:
+        fold = ""
+
     # create path if not exist
     dirt = f"{str(path)}/{str(problem_name)}/"
     try:
@@ -1194,25 +1408,24 @@ def write_dataframe_to_tsfile(
         pass  # raises os.error if path already exists
 
     # create ts file in the path
-    file = open(f"{dirt}{str(problem_name)}_transform.ts", "w")
+    file = open(f"{dirt}{str(problem_name)}{fold}.ts", "w")
 
     # write comment if any as a block at start of file
-    if comment:
+    if comment is not None:
         file.write("\n# ".join(textwrap.wrap("# " + comment)))
         file.write("\n")
+
     # begin writing header information
     file.write(f"@problemName {problem_name}\n")
-    file.write(f"@timeStamps {str(timestamp).lower()}\n")
+    file.write("@timestamps false\n")
     file.write(f"@univariate {str(univariate).lower()}\n")
+    file.write(f"@equalLength {str(equal_length).lower()}\n")
 
-    # write equal length or series length if provided
-    if equal_length:
-        file.write(f"@equalLength {str(equal_length).lower()}\n")
-    if series_length > 0:
+    if series_length > 0 and equal_length:
         file.write(f"@seriesLength {series_length}\n")
 
     # write class label line
-    if class_label:
+    if class_label is not None:
         space_separated_class_label = " ".join(str(label) for label in class_label)
         file.write(f"@classLabel true {space_separated_class_label}\n")
     else:
@@ -1221,23 +1434,21 @@ def write_dataframe_to_tsfile(
     # begin writing the core data for each case
     # which are the series and the class value list if there is any
     file.write("@data\n")
-    for case, value in itertools.zip_longest(data.iterrows(), class_value_list):
-        for dimension in case[1:]:  # start from the first dimension
-            # split the series observation into separate token
-            # ignoring the header and index
-            series = (
-                dimension[0]
-                .to_string(index=False, header=False, na_rep=missing_values)
-                .split("\n")
-            )
+    for case, value in itertools.zip_longest(data, class_value_list):
+        for dimension in case:
             # turn series into comma-separated row
-            series = ",".join(obsv for obsv in series)
+            series = ",".join(
+                [str(num) if not np.isnan(num) else missing_values for num in dimension]
+            )
             file.write(str(series))
             # continue with another dimension for multivariate case
             if not univariate:
                 file.write(":")
+        a = ":" if univariate else ""
         if value is not None:
-            file.write(f":{value}")  # write the case value if any
+            file.write(f"{a}{value}")  # write the case value if any
+        elif class_label is not None:
+            file.write(f"{a}{missing_values}")
         file.write("\n")  # open a new line
 
     file.close()
